<?php

namespace Controller;

use Exception\CustomException;
use Util\Instance\Service;
use Util\Utilities;
use Zeedhi\Framework\Controller\Simple;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\HTTP\Kernel;
use Util\Messages;
use Zeedhi\DevOps\Util\Utilities as DevOpsUtilities;


class BilletIssued extends Simple
{

    public function findBilletIssued(Request\Filter $request, Response $response) {
        $params = $request->getFilterCriteria()->getConditions();
        $filter = DevOpsUtilities::createQueryFiltered($params);
        try{

            $dataSet = Service::getBilletIssued()->findBilletIssued($filter);
            DevOpsUtilities::setDataSetDefault($request,$response,$dataSet);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function findBilletDetails(Request\Filter $request, Response $response) {
        $params = $request->getFilterCriteria()->getConditions();
        $nrlanctonf = DevOpsUtilities::getValueInFilterDataSourceByName($params,'NRLANCTONF');
        $cdfilial = DevOpsUtilities::getValueInFilterDataSourceByName($params,'CDFILIAL');
        try{

            $dataSet = Service::getBilletIssued()->findBilletDetails($nrlanctonf,$cdfilial);
            DevOpsUtilities::setDataSetDefault($request,$response,$dataSet);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }


    public function findProductDetails(Request\Filter $request, Response $response) {
        $params = $request->getFilterCriteria()->getConditions();
        $nrlanctonf = DevOpsUtilities::getValueInFilterDataSourceByName($params,'NRLANCTONF');
        $cdfilial = DevOpsUtilities::getValueInFilterDataSourceByName($params,'CDFILIAL');
        try{

            $dataSet = Service::getBilletIssued()->findProductDetails($nrlanctonf,$cdfilial);
            DevOpsUtilities::setDataSetDefault($request,$response,$dataSet);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function printBillet(Request\DataSet $request, Response $response)
    {
        try {
            $row = $request->getDataSet()->getRows()[0];
            $cdfilial = $row['CDFILIAL'];
            $nrlanbolrec = $row['NRLANBOLREC'];
            $cdreducont = $row['CDREDUCONT'];
            $dataSet = Service::getBilletIssued()->printBillet($cdfilial,$nrlanbolrec,$cdreducont);
            DevOpsUtilities::setDataSetDefault($request, $response, $dataSet);
        } catch (\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }

    }

} 