<?php
namespace Service;

use Doctrine\ORM\EntityManager;
use Util\InstanceProvider;
use Factory\OqueHaDeNovo as OqueHaDeNovoFactory;
use Util\Utilities;

class OqueHaDeNovo {
    /**
     * @var EntityManager
     */
    private $entityManager;
    /**
     * @var OqueHaDeNovoFactory
     */
    private $oqueHaDeNovoFactory;

    /**
     * Classification constructor.
     * @param EntityManager $entityManager
     * @param OqueHaDeNovoFactory $oqueHaDeNovoFactory
     */
    public function __construct(EntityManager $entityManager, OqueHaDeNovoFactory $oqueHaDeNovoFactory)
    {
        $this->entityManager = $entityManager;
        $this->oqueHaDeNovoFactory = $oqueHaDeNovoFactory;
    }

    public function findTreeGridSubject($parametros)
    {
        $customerNrParc = InstanceProvider::getUtilities()->getCurrentCustomer()->getNrparcnegocio();
        $assuntoId = $this->oqueHaDeNovoFactory->findAssuntoItensProductsClient($customerNrParc);
        $filterAssuntoItensProductsClient = 'AND 1=2';
        if (!empty( $assuntoId)) {
            $stringValues = "('" . implode("','",$assuntoId) . "')";
            $filterAssuntoItensProductsClient = 'AND ID IN '. $stringValues;
        }
        $data = $this->oqueHaDeNovoFactory->findTreeGridSubject($filterAssuntoItensProductsClient);
        $data['data'] = Utilities::fixParentsForTreeGrid($data['data'], 'ID', 'PAI_ID');
        return $data;
    }

    public function findItemProduct($dynamicFilter, $assuntoId)
    {
        $dtfiltroitemString = '';
        $assuntoIdString = '';
        $conditionsQuery = '';
        $customerNrParc = InstanceProvider::getUtilities()->getCurrentCustomer()->getNrparcnegocio();
        $dynamicFilter = Utilities::decodificarDynamicFilter($dynamicFilter);

        if (!empty($dynamicFilter["DTFILTROITEM"])) {
            $dates = $dynamicFilter["DTFILTROITEM"][0]["value"];
            $dateIni = $dates[0];
            $dateEnd = $dates[1];
            $dtfiltroitemString = " AND (  ( TO_DATE(DTFILTROITEM, 'DD/MM/YYYY HH24:MI:SS')  BETWEEN  TO_DATE('$dateIni 00:00:00', 'DD/MM/YYYY HH24:MI:SS') AND TO_DATE('$dateEnd 23:59:59', 'DD/MM/YYYY HH24:MI:SS') ) OR DTFILTROITEM IS NULL)";
            unset($dynamicFilter["DTFILTROITEM"]);
        }
        if (!empty($assuntoId)) {
            $filterTempAssunto = 'WHERE ASSUNTO_ID = '. $assuntoId;
            $assuntoIdString = "AND T.ID IN (SELECT ITEM_ID FROM OM_OQA_ITEM_ASSUNTO $filterTempAssunto)";
        }

        if (!empty($dynamicFilter)) {
            $conditionsQuery = $this->oqueHaDeNovoFactory->createDynamicFilter($dynamicFilter, 'WHERE 1=1',false);
        }
        $data =  $this->oqueHaDeNovoFactory->findItemProduct($dtfiltroitemString, $conditionsQuery, $assuntoIdString, $customerNrParc);
        $data['data'] = Utilities::fixParentsForTreeGrid($data['data'], 'TREE_ID');
        return $data;
    }
}
