/**
 * Gera as questoes em aberto sem respostas para o usuario responder posteriormente.
 * 
 * Observacoes:
 * - As questoes em aberto sao geradas na tabela EXECAVALIACAO.
 * - O usuario pode ser um parceiro de negocios (PARCNEGOCIO) ou um consumidor (CONSUMIDOR).
 * - As respostas serao gravadas posteriormente atraves da procedure AVL_GRAVA_RESPAVALIACAO.
 *
 * @author Ederson Melquiades da Cunha <ederson@teknisa.com>
 *
 * @param P_NRORG IN Organizacao logada
 * @param P_CDOPERADOR IN Operador logado
 * @param P_NRAVALIACAO IN Numero da avaliacao escolhida
 * @param P_NRPARCNEGOCIO IN Numero do parceiro de negocios que executara a avaliacao (opcional, passar '0' no lugar de NULL)
 * @param P_CDCLIENTE IN Codigo do cliente do consumidor que executara a avaliao (opcional, passar 'X' no lugar de NULL)
 * @param P_CDCONSUMIDOR IN Codigo do consumidor que executara a avaliacao (opcional, passar 'X' no lugar de NULL)
 *
 * @param P_DTEXECUCAOAVAL IN OUT Retorna a data/hora de execucao da avaliacao no formato 'YYYY/MM/DD HH24:MI:SS'
 *
 */

CREATE OR REPLACE
PROCEDURE AVL_GERA_EXECAVALIACAO (
   P_NRORG IN ORGANIZACAO.NRORG%TYPE, 
   P_CDOPERADOR IN OPERADOR.CDOPERADOR%TYPE,
   P_NRAVALIACAO IN COMBQUESTAVAL.NRAVALIACAO%TYPE,
   P_NRPARCNEGOCIO IN PARCNEGOCIO.NRPARCNEGOCIO%TYPE,
   P_CDCLIENTE IN CONSUMIDOR.CDCLIENTE%TYPE,
   P_CDCONSUMIDOR IN CONSUMIDOR.CDCONSUMIDOR%TYPE,
   P_DTEXECUCAOAVAL OUT EXECAVALIACAO.DTEXECUCAOAVAL%TYPE
) IS
   V_CDCONSUMIDOR CONSUMIDOR.CDCONSUMIDOR%TYPE;
   V_NRCOMBQUESTAVAL COMBQUESTAVAL.NRCOMBQUESTAVAL%TYPE;
   V_NREXECAVALIACAO NOVOCODIGO.NRSEQUENCIAL%TYPE;
   V_NRRESPAVALIACAO NOVOCODIGO.NRSEQUENCIAL%TYPE;
BEGIN
   SELECT MAX(NRCOMBQUESTAVAL)
     INTO V_NRCOMBQUESTAVAL
     FROM PARTICIPAVAL
    WHERE NRORG = P_NRORG
      AND NRAVALIACAO = P_NRAVALIACAO
      AND NRPARCEXECAVAL = P_NRPARCNEGOCIO;
   IF V_NRCOMBQUESTAVAL IS NULL THEN
      SELECT MAX(NRCOMBQUESTAVAL)
        INTO V_NRCOMBQUESTAVAL
        FROM COMBQUESTAVAL
       WHERE NRORG = P_NRORG
         AND NRAVALIACAO = P_NRAVALIACAO
         AND IDDEFAULT = 'S';
   END IF;
   IF P_DTEXECUCAOAVAL IS NULL THEN
      SELECT SYSDATE
        INTO P_DTEXECUCAOAVAL
        FROM DUAL;
   END IF;
   FOR CUR_QUESTAO IN (SELECT Q.NRQUESTAO, Q.NRTEMAANALISE, Q.TXCOMPLEMENTOQUESTAO
                         FROM QUESTAVALIACAO Q, COMBQUESTAVAL C
                        WHERE C.NRORG = P_NRORG
                          AND C.NRCOMBQUESTAVAL = V_NRCOMBQUESTAVAL
                          AND Q.NRORG = C.NRORG
                          AND Q.NRCOMBQUESTAVAL = C.NRCOMBQUESTAVAL) LOOP
      NOVO_CODIGO('EXECAVALIACAO', V_NREXECAVALIACAO, 1, P_NRORG);
      INSERT INTO EXECAVALIACAO  
         (NREXECAVALIACAO, NRORG, NRAVALIACAO,
          NRPARCAVALIADO, NRPARCEXECAVAL, NRQUESTAO,
          NRCOMBQUESTAVAL, DTEXECUCAOAVAL, IDATIVO,
          DTINCLUSAO, DTULTATU, NRORGINCLUSAO,
          CDOPERINCLUSAO, NRORGULTATU, CDOPERULTATU,
          NRFONTEDADOS, CDCLIENTE, CDCONSUMIDOR,
          NRTEMAANALISE, TXCOMPLEMENTOQUESTAO, IDORIGEMEXECAVAL)
      VALUES 
         (V_NREXECAVALIACAO, P_NRORG, P_NRAVALIACAO,
          NULL, P_NRPARCNEGOCIO, CUR_QUESTAO.NRQUESTAO,
          V_NRCOMBQUESTAVAL, P_DTEXECUCAOAVAL, 'S', 
          SYSDATE, SYSDATE, P_NRORG,
          P_CDOPERADOR, P_NRORG, P_CDOPERADOR, 
          NULL, P_CDCLIENTE, P_CDCONSUMIDOR,
          CUR_QUESTAO.NRTEMAANALISE, CUR_QUESTAO.TXCOMPLEMENTOQUESTAO, 'SISTEMA');
   END LOOP;
   COMMIT;
END;
/

-- VERSAO --
BEGIN ATUVEROBJ('AVL_GERA_EXECAVALIACAO','5.55.010'); END;
/
