import { Utilities } from '@/util/Utilities';
import { List, LoadingService, Modal, ModalService, Select, TextInput } from '@zeedhi/common';
import { Config, Http, MemoryDatasource, Metadata } from '@zeedhi/core';
import { TekGrid } from '@zeedhi/teknisa-components-common';
import { ValidaInscricao } from '@/util/ValidaInscricao';

export class CadastramentoAutomaticoPERController {
    public modalUploadArquivo!: Modal;

    public currentRow: any;

    public modalErrosArquivoPersonalizado!: Modal;

    public changeCampoSelect({ component }: any) {
        if (component.name === 'formLicencaUsoArqPersoform-IDTPIJURLUSO') {
            Metadata.getInstance<TextInput>('formLicencaUsoArqPersoform-NRINSJURLUSO').value = component.value;
        }
        if (component.name === 'formLicencaUsoArqPersoform-NMPAIS') {
            Metadata.getInstance<TextInput>('formLicencaUsoArqPersoform-CDPAIS').value = component.value;
            const UF = Metadata.getInstance<Select>('formLicencaUsoArqPersoform-UF');
            try {
                UF.datasource.setFilter({
                    CDPAIS: component.value,
                });
            } catch (e) {
                console.error(e);
            }
        }
        if (component.name === 'formLicencaUsoArqPersoform-UF') {
            Metadata.getInstance<TextInput>('formLicencaUsoArqPersoform-SGESTADO').value = component.value;
            const municipio = Metadata.getInstance<Select>('formLicencaUsoArqPersoform-NMMUNICIPIO');
            try {
                municipio.datasource.setFilter({
                    SGESTADO: component.value,
                });
            } catch (e) {
                console.error(e);
            }
        }
        if (component.name === 'formLicencaUsoArqPersoform-NMMUNICIPIO') {
            Metadata.getInstance<TextInput>('formLicencaUsoArqPersoform-CDMUNICIPIO').value = component.value;
        }
        if (component.name === 'formLicencaUsoArqPersoform-NMLINPCLIE') {
            Metadata.getInstance<TextInput>('formLicencaUsoArqPersoform-CDLINPCLIE').value = component.value.join(';');
        }
    }

    public downloadPlanilhaPadrao() {
        window.open(`${Config.endPoint}/downloadPlanilhaPadrao`);
    }

    public async openModalUploadArquivo() {
        this.modalUploadArquivo = await ModalService.createFromJson('cadAutomaticoPER/modal/modalUploadArquivo', true);
        this.modalUploadArquivo.show();
    }

    public closeModalUploadArquivo() {
        this.modalUploadArquivo.hide();
        this.modalUploadArquivo?.destroy();
    }

    public formattedData(response: any) {
        const arraysFromBackend: string[][] = response.data;

        // Nomes dos elementos que você deseja
        const elementNames = ['CDLOCALUSO', 'NMFANTLUSO', 'NMRAZSOCLUSO', 'IDTPIJURLUSO', 'NRINSJURLUSO', 'NRINSESTLUSO', 'NRINSMUNILUSO', 'NMCONTLUSO', 'NMDEPTOLUSO', 'DSENDELUSO', 'NMBAIRLUSO', 'NRCEPLUSO', 'NMPAIS', 'SGESTADO', 'NMMUNICIPIO', 'NRTELELUSO', 'NRRAMALLUSO', 'NRFAXLUSO', 'DSEMAILCOMLU', 'CDLINPCLIE'];

        const formattedData = arraysFromBackend.slice(1).map((array) => {
            const obj: { [key: string]: any } = {};
            elementNames.forEach((elementName, i) => {
              obj[elementName] = array[i];
            });
            return obj;
        });

        return formattedData;
    }

    public async leituraArquivoPersonalizado() {
        LoadingService.show();
        const grid = Metadata.getInstance<TekGrid>('gridExibicaoArquivo');
        const file = await Utilities.buildArrayToUpload('ARQUIVO');
        grid.datasource.data = [];
        (grid.datasource as MemoryDatasource).allData = [];
        if (file.length !== 0) {
            const response = await Http.post('/validaArquivoPersonalizado', {
                FILE: file,
            });
            const formattedData = this.formattedData(response);
            formattedData.forEach((element: any) => {
                const dados: any = {
                    CDLOCALUSO: element.CDLOCALUSO,
                    NMFANTLUSO: element.NMFANTLUSO,
                    NMRAZSOCLUSO: element.NMRAZSOCLUSO,
                    IDTPIJURLUSO: element.IDTPIJURLUSO,
                    NRINSJURLUSO: element.NRINSJURLUSO,
                    NRINSESTLUSO: element.NRINSESTLUSO,
                    NRINSMUNILUSO: element.NRINSMUNILUSO,
                    NMCONTLUSO: element.NMCONTLUSO,
                    NMDEPTOLUSO: element.NMDEPTOLUSO,
                    DSENDELUSO: element.DSENDELUSO,
                    NMBAIRLUSO: element.NMBAIRLUSO,
                    NRCEPLUSO: element.NRCEPLUSO,
                    NMPAIS: element.NMPAIS,
                    SGESTADO: element.SGESTADO,
                    NMMUNICIPIO: element.NMMUNICIPIO,
                    NRTELELUSO: element.NRTELELUSO,
                    NRRAMALLUSO: element.NRRAMALLUSO,
                    NRFAXLUSO: element.NRFAXLUSO,
                    DSEMAILCOMLU: element.DSEMAILCOMLU,
                    CDLINPCLIE: element.CDLINPCLIE,
               };
                grid.datasource.post(dados);
            });
            Utilities.showDialog('Atenção!', 'warning', 'As linhas em vermelho devem ser corrigidas no arquivo pois estão inválidas. Clique na linha para saber mais.');
        }
        grid.datasource.get();
        LoadingService.hide();
    }

    public colorirLinhasInválidas(row: any) {
        let erroEncontrado = false;
        const camposObrigatorios = [
            'CDLOCALUSO',
            'DSEMAILCOMLU',
            'DSENDELUSO',
            'IDTPIJURLUSO',
            'NMBAIRLUSO',
            'NMCONTLUSO',
            'NMDEPTOLUSO',
            'NMFANTLUSO',
            'NMMUNICIPIO',
            'NMPAIS',
            'NMRAZSOCLUSO',
            'NRCEPLUSO',
            'NRINSESTLUSO',
            'NRINSJURLUSO',
            'NRINSMUNILUSO',
            'SGESTADO',
        ];
        let campoInvalido: string | null = null;
        camposObrigatorios.forEach((campo: any) => {
            if (!row[campo]) {
                campoInvalido = campo;
            }
        });
        if (campoInvalido) {
            erroEncontrado = true;
        }
        if (row.CDLOCALUSO) {
			row.CDLOCALUSO = row.CDLOCALUSO.toString();
            if (row.CDLOCALUSO.length !== 4) {
                erroEncontrado = true;
            }
        }
        if (row.IDTPIJURLUSO) {
            if (!['F', 'J', 'I', 'O'].includes(row.IDTPIJURLUSO)) {
                erroEncontrado = true;
            }
        }
        if (row.IDTPIJURLUSO && row.NRINSJURLUSO) {
            if (row.IDTPIJURLUSO === 'F' && !ValidaInscricao.validateCPF(row.NRINSJURLUSO)) {
                erroEncontrado = true;
            }
            if (row.IDTPIJURLUSO === 'J' && !ValidaInscricao.validateCNPJ(row.NRINSJURLUSO)) {
                erroEncontrado = true;
            }
            if (row.IDTPIJURLUSO === 'I' && row.NRINSJURLUSO.length > 20) {
                erroEncontrado = true;
            }
            if (row.IDTPIJURLUSO === 'O' && row.NRINSJURLUSO.length > 5) {
                erroEncontrado = true;
            }
        }
        if (row.NRINSESTLUSO) {
            if (row.NRINSESTLUSO.length > 16) {
                erroEncontrado = true;
            }
        }
        if (row.NRCEPLUSO) {
            if (!/^\d+$/.test(row.NRCEPLUSO)) {
                erroEncontrado = true;
            }
        }
        if (row.NRRAMALLUSO) {
            if (!/^\d+$/.test(row.NRRAMALLUSO) && row.NRRAMALLUSO > 4) {
                erroEncontrado = true;
            }
        }
        if (row.NRFAXLUSO) {
            if (!/^\d+$/.test(row.NRFAXLUSO)) {
                erroEncontrado = true;
            }
        }

		if (row.NMBAIRLUSO) {
			if (row.NMBAIRLUSO.length && row.NMBAIRLUSO.length > 60) {
				erroEncontrado = true;
			}
		}
		if (row.NMFANTLUSO) {
			if (row.NMFANTLUSO.length && row.NMFANTLUSO.length > 60) {
				erroEncontrado = true;
			}
		}
		if (row.NMRAZSOCLUSO) {
			if (row.NMRAZSOCLUSO.length && row.NMRAZSOCLUSO.length > 50) {
				erroEncontrado = true;
			}
		}
		if (row.NRINSMUNILUSO) {
			if (row.NRINSMUNILUSO.length && row.NRINSMUNILUSO.length > 20) {
				erroEncontrado = true;
			}
		}
		if (row.NMCONTLUSO) {
			if (row.NMCONTLUSO.length && row.NMCONTLUSO.length > 50) {
				erroEncontrado = true;
			}
		}
		if (row.NMDEPTOLUSO) {
			if (row.NMDEPTOLUSO.length && row.NMDEPTOLUSO.length > 50) {
				erroEncontrado = true;
			}
		}
		if (row.DSENDELUSO) {
			if (row.DSENDELUSO.length && row.DSENDELUSO.length > 50) {
				erroEncontrado = true;
			}
		}
		if (row.NRTELELUSO) {
			if (row.NRTELELUSO.length && row.NRTELELUSO.length > 25) {
				erroEncontrado = true;
			}
		}
		if (row.NRRAMALLUSO) {
			if (row.NRRAMALLUSO.length && row.NRRAMALLUSO.length > 4) {
				erroEncontrado = true;
			}
		}
		if (row.NRFAXLUSO) {
			if (row.NRFAXLUSO.length && row.NRFAXLUSO.length > 25) {
				erroEncontrado = true;
			}
		}
		if (row.DSEMAILCOMLU) {
			if (row.DSEMAILCOMLU.length && row.DSEMAILCOMLU.length > 50) {
				erroEncontrado = true;
			}
		}
        if (row.CDLINPCLIE) {
            const cdlinpclie = `${row.CDLINPCLIE}`;
            if (cdlinpclie.includes(';')) {
                const cdlinpclieArray = cdlinpclie.split(';');
                if (cdlinpclieArray.length !== 2) {
                    erroEncontrado = true;
                }
                // cdlinpclieArray.forEach((item: any) => {
                //     if (item.length !== 20 || !/^\d+$/.test(item)) {
                //         erroEncontrado = true;
                //     }
                // });
            }
			// else if (cdlinpclie.length !== 20 || !/^\d+$/.test(cdlinpclie)) {
            //     erroEncontrado = true;
            // }
        }
        if (erroEncontrado) {
            return { cssClass: 'linhaGridVermelha' };
        }
        return undefined;
    }

    public saveDadosArquivoPersonalizado() {
        const grid = Metadata.getInstance<TekGrid>('gridExibicaoArquivo');
        const dados: any = (grid.datasource as MemoryDatasource).allData;
        let erroEncontrado = false;
        dados.forEach((dado: any) => {
            const erros = this.validaDadosArquivoPersonalizado(dado);
            erros.forEach((erro: any) => {
                if (erro) {
                    erroEncontrado = true;
                }
                dado.erroEncontrado = erroEncontrado;
            });
        });
        if (dados.length > 0) {
                LoadingService.show();
                Http.post('/saveCustomizationFiles', dados)
                .then((response) => {
                    if (response.data.row.length > 0) {
                        Utilities.showDialog('Erro!', 'error', `É necessário a correção das linhas em vermelho. Quantidade de linhas salvas: ${response.data.rowSave}`);
                        grid.datasource.data = response.data.row;
                        (grid.datasource as MemoryDatasource).allData = response.data.row;
                    } else {
                        grid.datasource.data = [];
                        (grid.datasource as MemoryDatasource).allData = [];
                        this.closeModalUploadArquivo();
                        Utilities.showSuccessMessage();
                        Metadata.getInstance<TextInput>('formLicencaUsoArqPersoform-IDTPIJURLUSO').value = null;
                        Metadata.getInstance<TextInput>('formLicencaUsoArqPersoform-NMPAIS').value = null;
                        Metadata.getInstance<TextInput>('formLicencaUsoArqPersoform-UF').value = null;
                        Metadata.getInstance<TextInput>('formLicencaUsoArqPersoform-NMMUNICIPIO').value = null;
                        Metadata.getInstance<TextInput>('formLicencaUsoArqPersoform-NMLINPCLIE').value = null;
                    }
                }).catch((error) => {
                    Utilities.showErrorMessage(error);
                }).finally(() => {
                    LoadingService.hide();
                });
        } else {
            Utilities.showDialog('Erro!', 'error', 'O grid deve conter pelo menos um registro.');
        }
    }

    public validaDadosArquivoPersonalizado(row: any) {
        const erros: any = [];
        const camposObrigatorios = [
            'CDLOCALUSO',
            'DSEMAILCOMLU',
            'DSENDELUSO',
            'IDTPIJURLUSO',
            'NMBAIRLUSO',
            'NMCONTLUSO',
            'NMDEPTOLUSO',
            'NMFANTLUSO',
            'NMMUNICIPIO',
            'NMPAIS',
            'NMRAZSOCLUSO',
            'NRCEPLUSO',
            'NRINSESTLUSO',
            'NRINSJURLUSO',
            'NRINSMUNILUSO',
            'SGESTADO',
        ];
        const camposInvalidos: string[] = [];
        camposObrigatorios.forEach((campo): any => {
            if (!row[campo]) {
                camposInvalidos.push(campo);
            }
        });
        const grid = Metadata.getInstance<TekGrid>('gridExibicaoArquivo');
        const nomeCamposInválidos: any = [];
        grid.columns.forEach((column: any) => {
            camposInvalidos.forEach((campo): any => {
                if (column.name === campo) {
                    nomeCamposInválidos.push(column.label);
                }
            });
        });
        if (nomeCamposInválidos.length > 0) {
            const concatenatedString = `Os campos ${nomeCamposInválidos.join(' e ')} são obrigatórios.`;
            erros.push({ mensagem: concatenatedString, row: 'OUTROS' });
        }
        if (row.CDLOCALUSO) {
			row.CDLOCALUSO = row.CDLOCALUSO.toString();
            if (row.CDLOCALUSO.length !== 4) {
                erros.push({ mensagem: 'O campo Local de Uso deve ter 4 caracteres.', row: 'CDLOCALUSO' });
            }
        }
        if (row.IDTPIJURLUSO) {
            if (!['F', 'J', 'I', 'O'].includes(row.IDTPIJURLUSO)) {
                erros.push({ mensagem: 'O campo Tipo de Organização só pode ter 4 valores: F, J, I, O.', row: 'IDTPIJURLUSO' });
            }
        }
        if (row.IDTPIJURLUSO && row.NRINSJURLUSO) {
            if (row.IDTPIJURLUSO === 'F' && !ValidaInscricao.validateCPF(row.NRINSJURLUSO)) {
                erros.push({ mensagem: 'Valor inválido do Número de Inscrição com Tipo de Organização F - Pessoa Física', row: 'NRINSJURLUSO' });
            } else if (row.IDTPIJURLUSO === 'J' && !ValidaInscricao.validateCNPJ(row.NRINSJURLUSO)) {
                erros.push({ mensagem: 'Valor inválido do Número de Inscrição com Tipo de Organização J - Pessoa Jurídica', row: 'NRINSJURLUSO' });
            } else if (row.IDTPIJURLUSO === 'I' && row.NRINSJURLUSO.length > 20) {
                erros.push({ mensagem: 'Valor inválido do Número de Inscrição com Tipo de Organização I - Inscrição Internacional', row: 'NRINSJURLUSO' });
            } else if (row.IDTPIJURLUSO === 'O' && row.NRINSJURLUSO.length > 5) {
                erros.push({ mensagem: 'Valor inválido do Número de Inscrição com Tipo de Organização O - Outros', row: 'NRINSJURLUSO' });
            }
        }
        if (row.NRINSESTLUSO) {
            if (row.NRINSESTLUSO.length > 16) {
                erros.push({ mensagem: 'O campo Inscrição Estadual deve conter até 16 caracteres', row: 'NRINSESTLUSO' });
            }
        }
        if (row.NRCEPLUSO) {
            if (!/^\d+$/.test(row.NRCEPLUSO)) {
                erros.push({ mensagem: 'O campo CEP deve conter apenas números', row: 'NRCEPLUSO' });
            }
        }
        if (row.NRRAMALLUSO) {
            if (!/^\d+$/.test(row.NRRAMALLUSO) && row.NRRAMALLUSO > 4) {
                erros.push({ mensagem: 'O campo Ramal deve conter apenas números e até 4 caracteres', row: 'NRRAMALLUSO' });
            }
        }
        if (row.NRFAXLUSO) {
            if (!/^\d+$/.test(row.NRFAXLUSO)) {
                erros.push({ mensagem: 'O campo Fax deve conter apenas números', row: 'NRFAXLUSO' });
            }
        }
        if (row.NMBAIRLUSO) {
            if (row.NMBAIRLUSO.length && row.NMBAIRLUSO.length > 60) {
                erros.push({ mensagem: 'O campo Bairro deve conter até 60 caracteres', row: 'NMBAIRLUSO' });
            }
        }
        if (row.NMFANTLUSO) {
            if (row.NMFANTLUSO.length && row.NMFANTLUSO.length > 60) {
                erros.push({ mensagem: 'O campo Nome Fantasia deve conter até 50 caracteres', row: 'NMFANTLUSO' });
            }
        }
        if (row.NMRAZSOCLUSO) {
            if (row.NMRAZSOCLUSO.length && row.NMRAZSOCLUSO.length > 50) {
                erros.push({ mensagem: 'O campo Razão Social deve conter até 50 caracteres', row: 'NMRAZSOCLUSO' });
            }
        }
        if (row.NRINSMUNILUSO) {
            if (row.NRINSMUNILUSO.length && row.NRINSMUNILUSO.length > 20) {
                erros.push({ mensagem: 'O campo Inscrição Municipal deve conter até 20 caracteres', row: 'NRINSMUNILUSO' });
            }
        }
		if (row.NMCONTLUSO) {
			if (row.NMCONTLUSO.length && row.NMCONTLUSO.length > 50) {
				erros.push({ mensagem: 'O campo Contato deve conter até 50 caracteres', row: 'NMCONTLUSO' });
			}
		}
		if (row.NMDEPTOLUSO) {
			if (row.NMDEPTOLUSO.length && row.NMDEPTOLUSO.length > 50) {
				erros.push({ mensagem: 'O campo Departamento deve conter até 50 caracteres', row: 'NMDEPTOLUSO' });
			}
		}
		if (row.DSENDELUSO) {
			if (row.DSENDELUSO.length && row.DSENDELUSO.length > 50) {
				erros.push({ mensagem: 'O campo Rua deve conter até 50 caracteres', row: 'DSENDELUSO' });
			}
		}
		if (row.NRTELELUSO) {
			if (row.NRTELELUSO.length && row.NRTELELUSO.length > 25) {
				erros.push({ mensagem: 'O campo Telefone deve conter até 25 caracteres', row: 'NRTELELUSO' });
			}
		}
		if (row.NRRAMALLUSO) {
			if (row.NRRAMALLUSO.length && row.NRRAMALLUSO.length > 4) {
				erros.push({ mensagem: 'O campo Ramal deve conter até 4 caracteres', row: 'NRRAMALLUSO' });
			}
		}
		if (row.NRFAXLUSO) {
			if (row.NRFAXLUSO.length && row.NRFAXLUSO.length > 25) {
				erros.push({ mensagem: 'O campo Fax deve conter até 25 caracteres', row: 'NRFAXLUSO' });
			}
		}
		if (row.DSEMAILCOMLU) {
			if (row.DSEMAILCOMLU.length && row.DSEMAILCOMLU.length > 50) {
				erros.push({ mensagem: 'O campo E-Mail (contato) deve conter até 50 caracteres', row: 'DSEMAILCOMLU' });
			}
		}
        if (row.CDLINPCLIE) {
			const cdlinpclie = `${row.CDLINPCLIE}`;
            if (cdlinpclie.includes(';')) {
                const cdlinpclieArray = cdlinpclie.split(';');
                if (cdlinpclieArray.length !== 2) {
                    erros.push({ mensagem: 'O campo Atribuído à Licença de Uso com mais de um registro deve separado por ;', row: 'CDLINPCLIE' });
                } else {
                    // cdlinpclieArray.forEach((item: any) => {
                    //     if (item.length !== 20 || !/^\d+$/.test(item)) {
                    //         erros.push({ mensagem: 'O campo Atribuído à Licença de Uso deve ter exatamente 20 caracteres e ser um número.', row: 'CDLINPCLIE' });
                    //     }
                    // });
                }
            }
			// else if (cdlinpclie.length !== 20 || !/^\d+$/.test(cdlinpclie)) {
            //     erros.push({ mensagem: 'O campo Atribuído à Licença de Uso deve ter exatamente 20 caracteres e ser um número.', row: 'CDLINPCLIE' });
            // }
        }
        return erros;
    }

    public fecharModalErrosArquivoPersonalizado() {
        this.modalErrosArquivoPersonalizado.hide();
        this.modalErrosArquivoPersonalizado?.destroy();
    }

    public async openModalErrosArquivoPersonalizado({ row }: any) {
        if (this.currentRow) {
            this.modalErrosArquivoPersonalizado?.destroy();
        }
        this.currentRow = row;
        this.modalErrosArquivoPersonalizado = await ModalService.createFromJson('cadAutomaticoPER/modal/modalErrosArquivoPersonalizado', true);
        this.modalErrosArquivoPersonalizado.show();
    }

    public mountedItems() {
        const groupList = Metadata.getInstance<List>('listModalErrosArquivoPersonalizado');
        const erros = this.validaDadosArquivoPersonalizado(this.currentRow);
        erros.forEach((erro: any) => {
            groupList.childrenInstances.forEach((item: any) => {
                if (item.name === erro.row) {
                    item.subtitle = erro.mensagem;
                }
            });
        });
    }
}
