import { Http, Metadata, Validation } from '@zeedhi/core';
import { ITekGridLayout, TekRestDatasource, TekTreeGrid } from '@zeedhi/teknisa-components-common';
import { Utilities } from '@/util/Utilities';
import { LoadingService } from '@zeedhi/common';

const invalidDate = () => {
    const message = 'É necessário que seja um intervalo entre duas datas';
    return (value: string[]) => {
        if (value.length === 1) {
            return message;
        }
        return true;
    };
};

const intervalDate = () => {
    const message = 'O intervalo deve ser de no máximo 1 ano';
    return (value: string[]) => {
        if (value.length === 2) {
            const [day1, month1, year1] = value[0].split('/').map(Number);
            const [day2, month2, year2] = value[1].split('/').map(Number);
            const days1 = year1 * 365 + month1 * 30 + day1;
            const days2 = year2 * 365 + month2 * 30 + day2;
            const intervalInDays = Math.abs(days2 - days1);
            if (intervalInDays > 365) {
                return message;
            }
        }
        return true;
    };
};

Validation.register('invalidDate', invalidDate);
Validation.register('intervalDate', intervalDate);

export class OqueHaDeNovoController {
    public baseUrl = '';

    public async filterOnMounted(params: any) {
        const oqueHaDeNovoTreeGrid = params.component;
		if (oqueHaDeNovoTreeGrid.datasource instanceof TekRestDatasource) {
			const dynamicFilter = {
				DTFILTROITEM: [
					{
						operation: 'BETWEEN',
						relation: 'AND',
						value: Utilities.getDates(),
					},
				],
			};
			const defaultLayout: ITekGridLayout = {
				name: 'Padrão',
				dynamicFilter,
			};
			oqueHaDeNovoTreeGrid.layoutOptions?.updateDefaultLayout(defaultLayout);
		}
    }

    public async filterAssunto({ row }: any) {
        const oqueHaDeNovoTreeGrid = Metadata.getInstance<TekTreeGrid>('oqueHaDeNovoTreeGrid');
		try {
			oqueHaDeNovoTreeGrid.datasource.setFilter({
				ASSUNTO_ID: row.ID,
			});
		} catch (e) {
			console.error(e);
		}
    }

    public checkItemDocument({ row }: any) {
		if (row.IS_PRODUCT === 'N' && row.FILENAME !== null) {
			return true;
		}
        return false;
    }

	public async downloadDocumentItem({ row }: any) {
        LoadingService.show();
        try {
            const response = await Http.post('/downloadDocumentItem', {
                ID: row.ID,
            });
            window.open(response.data);
            LoadingService.hide();
        } catch (e) {
            Utilities.showErrorMessage(e);
            LoadingService.hide();
        }
        LoadingService.hide();
    }

    public teste({ component }: any) {
        console.log(component);
    }
}
